local mod = ...



function IsBurnPart(obj, comp)

    local isBurn = true

    -- Get the building name
    local compBuilding = obj:findFirstObjectWithComponentUp("COMP_BUILDING")
    local nameBuilding = ""
    local typeBuilding = ""
    if (compBuilding) then
        nameBuilding = compBuilding.AssetBuilding.Name
        typeBuilding = compBuilding.AssetBuilding.BuildingType
    end

    -- Exclude certain building parts from being burned
    if obj.Name == "TempVillageCenterPart" then isBurn = false
    elseif obj.Name == "VillageCenter" then isBurn = false
    elseif obj.Name == "OutlawCamp.Part" then isBurn = false
    elseif obj.Name == "FireStarter.Part" then isBurn = false
    elseif obj.Name == "Alert.Part" then isBurn = false
    elseif string.sub(obj.Name, 1, 6) == "PigPen" then isBurn = false
    elseif string.sub(obj.Name, 1, 6) == "PigYar" then isBurn = false
    elseif string.sub(obj.Name, 1, 7) == "Pasture" then isBurn = false
    elseif string.sub(obj.Name, 1, 8) == "SheepYar" then isBurn = false
    elseif string.sub(obj.Name, 1, 8) == "SheepPen" then isBurn = false
    elseif string.sub(obj.Name, 1, 9) == "CattleYar" then isBurn = false
    elseif string.sub(obj.Name, 1, 9) == "CattlePen" then isBurn = false
    elseif string.sub(obj.Name, 1, 9) == "CattlePas" then isBurn = false

    -- Exclude certain buildings
    elseif nameBuilding == "BUILDING_TERRAIN_NAME" then isBurn = false
    elseif nameBuilding == "FISHPOND_NAME" then isBurn = false
    elseif nameBuilding == "WOODEN_BRIDGE" then isBurn = false
    elseif nameBuilding == "STONE_BRIDGE" then isBurn = false
    elseif nameBuilding == "IRON_DEPOSIT" then isBurn = false
    elseif nameBuilding == "GOLD_DEPOSIT" then isBurn = false
    elseif nameBuilding == "MARBLE_DEPOSIT" then isBurn = false
    elseif nameBuilding == "QUARTZ_DEPOSIT" then isBurn = false
    elseif nameBuilding == "STONE_DEPOSIT" then isBurn = false

    -- Exclude certain building types
    elseif typeBuilding == "WALL" then isBurn = false

    -- Exclude if not built or not visible
    elseif not comp:isVisibleWhenBuilt() then isBurn = false
    elseif not comp:getIsBuilt() then isBurn = false
    elseif not obj.Active then isBurn = false

    end

    return isBurn

end



function RemoveAlert(objDelete)

    -- -- Testing
    -- objDelete:forEachComponent(function(compDelete)
    --     mod:log(tostring(compDelete:getComponentType()))
    -- end)

    -- Remove resource depot (clears inventory in the Resource toolbar) - must be done before removing resource generator
    local compRes = objDelete:getComponent("COMP_RESOURCE_DEPOT")
    if compRes then
        -- compRes.OutputInventory.Inventory.Collection[1].Quantity = 0
        objDelete:removeComponent(compRes)
        -- mod:log("Removed resource depot component")
    end

    -- Remove resource generator (next time resource tracker refreshes, no alert will be shown)
    local compGen = objDelete:getComponent("COMP_RESOURCE_GENERATOR")
    if compGen then
        -- compGen:setMaxQuantity(0, true)
        -- compGen.MaxQuantity = 0
        objDelete:removeComponent(compGen)
        -- mod:log("Removed resource generator component")
    end

    -- local compPart = objDelete:getComponent("COMP_BUILDING_PART")
    -- if compPart then
    --     objDelete:removeComponent(compPart) -- causes a crash
    --     mod:log("Removed building part component")
    -- end

    objDelete.Active = false -- ensures it will not be a subject of burning itself
    -- objDelete:destroy() -- causes crash when you right-click the resource icon again, seemingly due to removal of the building part component

end




-- ************************************************************************************************
-- Component: Controller --------------------------------------------------------------------------
-- ************************************************************************************************

-- Component
local COMP_FIRE_CONTROLLER = {
	TypeName = "COMP_FIRE_CONTROLLER",
	ParentType = "COMPONENT",
	Properties = {
    }
}


function COMP_FIRE_CONTROLLER:onEnabled()

    -- -- TESTING ONLY
    -- for i = 1, 3 do
    --     local posStart = {720.476379, 5.056122, 656.991211}
    --     if posStart then

    --         -- Create object
    --         self:getLevel():createObject("PREFAB_FIRE_START", posStart)

    --         -- Notification
    --         local compVillager = self:getLevel():getComponentManager("COMP_VILLAGER"):getFirstEnabled()
    --         local jobAsset = nil
    --         local jobWorkplace = nil
    --         if (compVillager) then

    --             local job = compVillager:getJobInstance()
    --             if (job) then
    --                 jobAsset = job.AssetJob
    --                 jobWorkplace = job.Workplace
    --             end

    --             compVillager:changeJob("JOB_FIRE_EVENT_START")

    --             if (job) then
    --                 if (jobWorkplace) then
    --                     compVillager:changeJob(jobAsset, jobWorkplace)
    --                 else
    --                     compVillager:changeJob(jobAsset)
    --                 end
    --             else
    --                 compVillager:changeJob(nil)
    --             end

    --         end -- Villager found to trigger the notification

    --     end -- Position found for starting the fire
    -- end


    -- set up daily event
    local compMainGameLoop = self:getLevel():find("COMP_MAIN_GAME_LOOP")
    event.register(self, compMainGameLoop.ON_NEW_DAY, function()


        -- -- Find out current fire count
        -- local countFire = 0
        -- self:getLevel():getComponentManager("COMP_FIRE"):getAllEnabledComponent():forEach(function(compFire)
        --     countFire = countFire + 1
        -- end)


        -- -- Clean up builders huts in case they were not properly converted due to previous mod bugs
        -- if countFire == 0 then
        --     local obj = self:getLevel():find("TempVillageCenterPart")
        --     if obj then
        --         self:updateFireBrigade(obj, 2000, 2)
        --     end
        -- end


        -- Manage fire resource generators (alerts)
        local listDelete = {}
        self:getLevel():getComponentManager("COMP_FIRE_ALERT"):getAllEnabledComponent():forEach(function(compAlert)

            -- Decrement timer
            compAlert.Timer = compAlert.Timer - 1

            -- mod:log("Alert timer = " .. compAlert.Timer)

            -- Disable alert if timer has run down (deletion causes a crash when using resource tracker)
            if compAlert.Timer == 0 then
                table.insert(listDelete, compAlert:getOwner():getId())
            end

        end)

        if #listDelete > 0 then
            for i = 1, #listDelete do

                local objDelete = self:getLevel():find(listDelete[i])
                if objDelete then

                    RemoveAlert(objDelete)

                end
            end
        end


        -- Find out current population
        local countPop = 0
        self:getLevel():getComponentManager("COMP_VILLAGER"):getAllEnabledComponent():forEach(function(compVill)
            countPop = countPop + 1
        end)


        -- Chance that a fire will start
        local chance = math.random()
        if (chance < (1/100)*(countPop/100)) and countPop >= 30 then
        -- if chance < 0.50 then -- FOR TESTING

            -- Record position of all building parts
            local listPos = {}
            self:getLevel():getComponentManager("COMP_BUILDING_PART"):getAllEnabledComponent():forEach(function(compPart)

                local objPart = compPart:getOwner()
                if objPart then

                    local isBurn = IsBurnPart(objPart, compPart)
                    if isBurn then
                        local posPart = objPart:getGlobalPosition()
                        table.insert(listPos, posPart)
                    end

                end

            end)

            -- Pick a random index from the table
            local posStart = nil
            if #listPos > 0 then
                local i = math.random(1, #listPos)
                posStart = listPos[i]
            end

            -- Start the fire
            if posStart then

                -- Create object
                self:getLevel():createObject("PREFAB_FIRE_START", posStart)

                -- Notification
                local compVillager = self:getLevel():getComponentManager("COMP_VILLAGER"):getFirstEnabled()
                local jobAsset = nil
                local jobWorkplace = nil
                if (compVillager) then

                    local job = compVillager:getJobInstance()
                    if (job) then
                        jobAsset = job.AssetJob
                        jobWorkplace = job.Workplace
                    end

                    compVillager:changeJob("JOB_FIRE_EVENT_START")

                    if (job) then
                        if (jobWorkplace) then
                            compVillager:changeJob(jobAsset, jobWorkplace)
                        else
                            compVillager:changeJob(jobAsset)
                        end
                    else
                        compVillager:changeJob(nil)
                    end

                end -- Villager found to trigger the notification

            end -- Position found for starting the fire

        end -- if chance

    end) -- end daily event


end



function COMP_FIRE_CONTROLLER:updateFireBrigade(objFireOrigin, range, command)

    self.builderList = {}

    -- Find vanilla builder workshops
    self:findBuilder(objFireOrigin, range, "BuilderWorkshopCorePart")

    -- Find Frontier Essentials mod workshops
    if (foundation.isModLoaded("60041e79-3595-49fc-8ce3-0a3c6d805be2")) then
        self:findBuilder(objFireOrigin, range, "BuilderPart")
    end


    if self.builderList then

        for i = 1, #self.builderList do

            local objBuilder = self.builderList[i]
            if objBuilder then

                -- mod:log("Builder " .. tostring(objBuilder:getId()))

                -- Find fires within range
                local countFire = 0
                local posBuilder = objBuilder:getGlobalPosition()
                self:getLevel():getComponentManager("COMP_FIRE"):getAllEnabledComponent():forEach(function(compFire)

                    local objFire = compFire:getOwner()

                    local posFire = objFire:getGlobalPosition()
                    local dist = math.sqrt( (posBuilder.x - posFire.x)^2 + (posBuilder.z - posFire.z)^2 )

                    if dist <= range then
                        countFire = countFire + 1
                    end

                end)

                -- mod:log("Fires within range " .. countFire)


                -- Switch Builder to Fire Brigade
                if command == 1 and countFire > 0 then

                    local compWork = objBuilder:getComponent("COMP_WORKPLACE")
                    if compWork then

                        -- change jobs
                        local listWorker = compWork.AssignedWorkerList
                        if listWorker then
                            for n = 1, #listWorker do
                                listWorker[n]:changeJob("JOB_FIRE_WATER")
                            end
                        end

                        -- change function
                        compWork:setWorkplaceData("BUILDING_FUNCTION_FIRE_WATER")

                        -- mod:log("Set fire brigade function and jobs.")


                    end -- workplace exists


                -- Switch Fire Brigade back to Builder
                elseif command == 2 and countFire <= 1 then -- one fire may remain, that called this function

                    local compWork = objBuilder:getComponent("COMP_WORKPLACE")
                    if compWork then

                        -- change jobs
                        local listWorker = compWork.AssignedWorkerList
                        if listWorker then
                            for n = 1, #listWorker do
                                listWorker[n]:changeJob("BUILDER")
                            end
                        end

                        -- change function
                        compWork:setWorkplaceData("BUILDING_FUNCTION_BUILDER_WORKSHOP")

                        -- mod:log("Set builder function and jobs.")


                    end -- workplace exists

                end

            end

        end

    end

end -- registerFire



function COMP_FIRE_CONTROLLER:findBuilder(objFire, range, objName)


    -- Set variables
    local posFire = objFire:getGlobalPosition()

    -- Make a list of builders workshops within range
    local listBuilderWorkshop = {}
    self:getLevel():find(objName, listBuilderWorkshop)

    if listBuilderWorkshop then
        for i = 1, #listBuilderWorkshop do

            local objWork = listBuilderWorkshop[i]
            local posWork = objWork:getGlobalPosition()
            local compPart = objWork:getComponent("COMP_BUILDING_PART")
            local dist = math.sqrt( (posFire[1] - posWork[1])^2 + (posFire[3] - posWork[3])^2 )

            if (dist <= range) and objWork.Active and compPart:getIsBuilt() then

                -- Record the workshop
                table.insert(self.builderList, objWork)

            end -- within range, active and built

        end -- loop through builder objects

    end -- there is at least one builder workshop

end



-- Register
mod:registerClass(COMP_FIRE_CONTROLLER)



-- Attach component
mod:registerPrefabComponent("PREFAB_MANAGER", { DataType = "COMP_FIRE_CONTROLLER" } )


